% Units
% Energy : eV
% Time : s
% Temperature : K
% Charge : C
% Current : A

% ------------------------------------------------------
% close all;
% clear all;
% clc;

% ---------------------------
% Constants

% electron charge (C)
e = 1.602176565E-19;

% Planck constant (eV*s)
h = 4.135667516E-15;

% current unit : 2*e/hbar
C0 = 2*pi*2*e/h;
% ===========================

% ---------------------------
% Model

% number of modes
M = 4;

% couplings to leads
gamma = 1E-3; % graphene electrode

% energy spacing
dE = -1E-2; % energy spacing of side states
DE = 1; % energy spacing of centre states

% hopping coupling
J = 0.1;

% temperature
T = 300;

% bias voltage
Vsd = 5E-2;

% chemical potentials
muL = Vsd/2;
muR = -Vsd/2;

% gate voltage
% Vg

% lever arm
LAS = 0.2;
LAC = 1;

% parameters of the Redfield equation
RE = struct('M',M, 'Jmatrix',[], 'gammaL',[], 'gammaR',[],...
    'T',T, 'muL',muL, 'muR',muR,...
    'EE',[], 'ES',[],...
    'GinL',[], 'GinR',[], 'GoutL',[], 'GoutR',[],...
    'REmatrix',[], 'REvector',[]);
% ===========================

% ---------------------------
% Matrices

% Hamiltonian
H0 = zeros(M,M);
H0(1,2) = J; H0(2,1) = J;
H0(2,3) = J; H0(3,2) = J;
H0(3,4) = J; H0(4,3) = J;
H0(4,1) = J; H0(1,4) = J;

% energy spacing
ES = zeros(M,M);
ES(1,1) = 0.5; ES(3,3) = -0.5;
EC = zeros(M,M);
EC(2,2) = 0.5; EC(4,4) = -0.5;

% gate
gateS = zeros(M,M);
gateS(1,1) = 0.5; gateS(3,3) = -0.5;
gateC = zeros(M,M);
gateC(2,2) = 1; gateC(4,4) = 1;
% ===========================

% ---------------------------
Vgmin = -0.1;
Vgmax = 0.1;
NVg = 201;
Vgaxis = Vgmin + (Vgmax-Vgmin)*(0:NVg-1)/(NVg-1);
Cur = zeros(1,NVg);

for iVg = 1:NVg
    Vg = Vgaxis(iVg);
    
    H = H0 + dE.*ES + DE.*EC + LAS.*Vsd.*gateS + LAC.*Vg.*gateC;
    
    % J-matrix (eV)
    RE.Jmatrix = H;
    
    % gamma_L matrix (eV)
    RE.gammaL = zeros(1,M); RE.gammaL(1) = gamma;
    
    % gamma_R matrix (eV)
    RE.gammaR = zeros(1,M); RE.gammaR(3) = gamma;
    
    % diagonalization
    [RE.EE,RE.ES]=diagonalization(RE.Jmatrix);
%     RE_EE = RE.EE
%     RE_ES = RE.ES
    
    % Gamma matrices
    [RE.GinL,RE.GinR,RE.GoutL,RE.GoutR] = Gamma_matrices(RE);
%     GinL = RE.GinL
%     GinR = RE.GinR
%     GoutL = RE.GoutL
%     GoutR = RE.GoutR
    
    % Redfield equation
    [RE.REmatrix,RE.REvector] = Redfield(RE);
%     REmatrix = RE.REmatrix
%     REvector = RE.REvector
    
    % dephasing
    DPrate = 0E-2;
%     RE.REmatrix = RE.REmatrix - DPrate.*eye(M^2);
%     for i = 1:M
%         for j = 1:M
%             u = M*(i-1) + j;
%             if i~=j
%                 RE.REmatrix(u,u) = RE.REmatrix(u,u) - DPrate;
%             end
%         end
%     end
    for m = 1:M
        for n = 1:M
            u = M*(m-1) + n;
            for mm = 1:M
                for nn = 1:M
                    v = M*(mm-1) + nn;
                    for i = 1:M
                        for j = 1:M
                            if i~=j
                                RE.REmatrix(u,v) = RE.REmatrix(u,v)...
                                    - DPrate*RE.ES(i,m)*RE.ES(j,n)'*RE.ES(i,mm)'*RE.ES(j,nn);
                            end
                        end
                    end
                end
            end
        end
    end
    
    % steady state
    SS = linsolve(RE.REmatrix,-RE.REvector);
%     dS = RE.REmatrix*SS + RE.REvector
    
    % current
    current = current_Redfield(RE,SS);
    
    Cur(iVg) = C0*abs(current);
    
    iVg
end
% ===========================

% ---------------------------
% figure
hold on
plot(Vgaxis,1E9.*Cur,'-r')
% axis([Jmin Jmax 0 8E1])
% set(gca,'XTick',(-0.01:0.005:0.01))
% set(gca,'YTick',(0:2E1:8E1))
set(gca,'YScale','log')
% axis([Jmin Jmax 1E0 1E2])
% set(gca,'XTick',(-0.01:0.005:0.01))
% set(gca,'YTick',10.^(0:1:2))
set(gca,'LineWidth',1)
set(gca,'FontSize',24)
set(gca,'FontName','Times New Roman')
set(gcf, 'Position', [100 100 600 450])
xlabel('Energy (eV)')
ylabel('Current (nA)')
% ===========================
% ======================================================



%======================================================
kB = 8.617332E-5;
iVg = 101;
Cmin = Cur(1,iVg);
CMOSFET = Cmin.*exp(abs(Vgaxis)./(kB*T));

hold on
plot(Vgaxis,1E9.*CMOSFET,'-r')
%------------------------------------------------------